import { useState, useMemo } from 'react';
import { useSyncExternalStore } from 'use-sync-external-store';

function create$1(createState) {
  let state;
  const listeners = new Set();
  const setState = (partial, replace) => {
    const nextState = typeof partial === "function" ? partial(state) : partial;
    if (nextState !== state) {
      const previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach((listener) => listener(state, previousState));
    }
  };
  const getState = () => state;
  const subscribeWithSelector = (listener, selector = getState, equalityFn = Object.is) => {
    let currentSlice = selector(state);
    function listenerToAdd() {
      const nextSlice = selector(state);
      if (!equalityFn(currentSlice, nextSlice)) {
        const previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }
    listeners.add(listenerToAdd);
    return () => listeners.delete(listenerToAdd);
  };
  const subscribe = (listener, selector, equalityFn) => {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }
    listeners.add(listener);
    return () => listeners.delete(listener);
  };
  const destroy = () => listeners.clear();
  const api = { setState, getState, subscribe, destroy };
  state = createState(setState, getState, api);
  return api;
}

function create(createState) {
  const api = typeof createState === "function" ? create$1(createState) : createState;
  const useStore = (selector = api.getState, equalityFn = Object.is) => {
    const [err, setErr] = useState(null);
    if (err) {
      setErr(null);
      throw err;
    }
    const getSnapshot = useMemo(() => {
      let lastSnapshot;
      let lastSlice;
      return () => {
        let slice = lastSlice;
        const snapshot = api.getState();
        if (lastSnapshot === void 0 || !Object.is(lastSnapshot, snapshot)) {
          try {
            slice = selector(snapshot);
            if (lastSlice !== void 0 && equalityFn(lastSlice, slice)) {
              slice = lastSlice;
            } else {
              lastSlice = slice;
            }
          } catch (error) {
            setErr(error);
          }
          lastSnapshot = snapshot;
        }
        return slice;
      };
    }, [selector, equalityFn]);
    return useSyncExternalStore(api.subscribe, getSnapshot);
  };
  Object.assign(useStore, api);
  return useStore;
}

export { create as default };
