'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var useSyncExternalStore = require('use-sync-external-store');

function create$1(createState) {
  var state;
  var listeners = new Set();

  var setState = function setState(partial, replace) {
    var nextState = typeof partial === 'function' ? partial(state) : partial;

    if (nextState !== state) {
      var _previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach(function (listener) {
        return listener(state, _previousState);
      });
    }
  };

  var getState = function getState() {
    return state;
  };

  var subscribeWithSelector = function subscribeWithSelector(listener, selector, equalityFn) {
    if (selector === void 0) {
      selector = getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var currentSlice = selector(state);

    function listenerToAdd() {
      var nextSlice = selector(state);

      if (!equalityFn(currentSlice, nextSlice)) {
        var _previousSlice = currentSlice;
        listener(currentSlice = nextSlice, _previousSlice);
      }
    }

    listeners.add(listenerToAdd);
    return function () {
      return listeners.delete(listenerToAdd);
    };
  };

  var subscribe = function subscribe(listener, selector, equalityFn) {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }

    listeners.add(listener);
    return function () {
      return listeners.delete(listener);
    };
  };

  var destroy = function destroy() {
    return listeners.clear();
  };

  var api = {
    setState: setState,
    getState: getState,
    subscribe: subscribe,
    destroy: destroy
  };
  state = createState(setState, getState, api);
  return api;
}

function create(createState) {
  var api = typeof createState === 'function' ? create$1(createState) : createState;

  var useStore = function useStore(selector, equalityFn) {
    if (selector === void 0) {
      selector = api.getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var _useState = react.useState(null),
        err = _useState[0],
        setErr = _useState[1];

    if (err) {
      setErr(null);
      throw err;
    }

    var getSnapshot = react.useMemo(function () {
      var lastSnapshot;
      var lastSlice;
      return function () {
        var slice = lastSlice;
        var snapshot = api.getState();

        if (lastSnapshot === undefined || !Object.is(lastSnapshot, snapshot)) {
          try {
            slice = selector(snapshot);

            if (lastSlice !== undefined && equalityFn(lastSlice, slice)) {
              slice = lastSlice;
            } else {
              lastSlice = slice;
            }
          } catch (error) {
            setErr(error);
          }

          lastSnapshot = snapshot;
        }

        return slice;
      };
    }, [selector, equalityFn]);
    return useSyncExternalStore.useSyncExternalStore(api.subscribe, getSnapshot);
  };

  Object.assign(useStore, api);
  return useStore;
}

exports['default'] = create;
